package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: Functor.java 65 2008-07-02 19:47:59Z toni $

/**
 * Basically a wrapper inspired by {@link java.util.concurrent.Callable}
 * and {@link java.lang.Runnable}.
 * Allows exported methods to explicitly declare a parameter as a
 * function to be applied to an object.
 * <br />
 * Defines a single method, {@link #apply}.
 * <br />
 * Due to Rhino's code adaptations, passing a JavaScript function
 * where a {@link Functor} is expected works transparently, e.g.:
 * <code>lang.for_each(arr, function() { ... });</code>
 * @see net.outlyer.plugins.utils.LanguageExtensions
 * @see net.outlyer.plugins.utils.LanguageExtensions#for_each(Object[], Functor)
 * 
 * @since {@link API#REVISION} 1
 */
public interface Functor<ResultType, ParamType> {
    /**
     * Apply the function to object p and return a ResultType object.
     */
    ResultType apply(final ParamType p);
}
