package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: PluginObject.java 62 2008-07-01 00:27:28Z toni $

import java.util.HashSet;
import java.util.Set;

/**
 * Default plugin object, adds a "features" field intended to describe
 * the plugin features, and execution() which provides the number of times
 * the script has been executed.
 */
public class PluginObject extends BasePluginObject {
    /**
     * Plugins can describe which features they provide by adding
     * elements to this field. Possible values are application-specific.
     * Default: empty
     * @see #properties
     */
    public Set<String> features;

    {
        features = new HashSet<String>();
    }

    /**
     * Obtains the number of the current execution
     * @return Current execution number
     * @see Sandbox#getExecution()
     */
    public int execution() {
        return getSandbox().getExecution();
    }

    /**
     * {@inheritDoc}
     */
    @Override public Object clone() throws CloneNotSupportedException {
        final PluginObject po = (PluginObject) super.clone();
        po.features = new HashSet<String>(this.features);
        return po;
    }
    
    
}
