package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: PluginProperties.java 68 2008-07-05 14:01:44Z toni $

/**
 * Common properties of the plugin object. All of them
 * take sane defaults.
 */
// Can't be package-protected or its fields won't be modifiable from
// js code!
// This limitation appears to apply only to fields, not methods
public class PluginProperties extends SandboxAccessorImpl {

    // This class isn't meant to be used outside of this package, but must be
    // public (see comment above)
    PluginProperties(){}
    
    /**
     * pluggablejs Api Version against which this plugin was written.
     * Default: 1
     * @see API#VERSION
     */
    public int apiVersion = 1;

    /**
     * Type of this plugin, application-specific.
     * Default: "" (no type)
     * @see PluginLocator#getAll()
     * @see PluginLocator#getFirst()
     */
    public String type;

    /**
     * Name of this plugin.
     * Default: same as filename (including extension but not path)
     * @see PluginLocator#getAll()
     * @see PluginLocator#getFirst()
     */
    public String name;

    /**
     * @deprecated The use of this field is yet to be defined
     */
    public Object properties; // FIXME: To be defined

    @Override public String toString() {
        return String.format("PluginProperties{ name=%s, type=%s, apiVersion=%d }",
                             name, type, apiVersion);
    }
}
