package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: SandboxAccessor.java 62 2008-07-01 00:27:28Z toni $

/**
 * Optional interface for exported objects (see
 * {@link net.outlyer.plugins.sandboxing.PluginEnvironment#exportGlobalObject}),
 * it enables exported objects to retrieve the sandbox object in which they're
 * being executed.
 * 
 * Implementors must contain a field like
 * <pre>
 * public Object sandboxGetter = null;
 * </pre>
 * 
 * sandboxGetter MUST be public, it can be of class Object or
 * Callable&lt;Sandbox&gt;, but shouldn't be used directly, instead, if required,
 * use SandboxAccessorImpl#getSandbox(SandboxAccessor) to retrieve it's
 * value, or inherit from SandboxAccessorImpl to be able to use
 * SandboxAccessorImpl#getSandbox().
 * <br />
 * Implementation notes: It will be dynamically initialised, no user
 * initialisation is required.
 *
 * The convenience default implementation, {@link SandboxAccessorImpl} can
 * be used as a base class if that's possible, to hide such requirement.
 */
public interface SandboxAccessor {
    /**
     * Obtains the associated sanbdox.
     * Implementors can use {@link SandboxAccessor#getSandbox(SandboxAccessor)}.
     * @return The sandbox
     */
    public Sandbox getSandbox();
}
