package net.outlyer.plugins.utils;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: LanguageExtensions_V1.java 68 2008-07-05 14:01:44Z toni $

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import javax.script.ScriptException;
import net.outlyer.plugins.PluginException;
import net.outlyer.plugins.PluginReader;
import net.outlyer.plugins.Sandbox;
import net.outlyer.plugins.SandboxAccessorImpl;

/**
 * Initial version of LanguageExtensions.
 * @see LanguageExtensions
 */
class LanguageExtensions_V1 extends SandboxAccessorImpl {
    /**
     * ScriptEngine's eval.
     * The built-in eval() appears to be less-powerful, in general they're
     * equivalent, but if the eval'ed string is obtainer from input
     * <code>eval()</code> appears to ignore it, while
     * <code>LanguageExtensions.eval()</code> does eval it.
     * @param code Code to evaluate
     * @return Eval's result
     */
    public Object eval(final String code) {
        try {
            return getSandbox().getCurrentEngine().eval(code);
        }
        catch (final ScriptException e) {
            // FIXME: Handle better
            System.err.println("Exception: " + e.getMessage());
        }
        return null;
    }

    /**
     * Load an external file
     * @deprecated Not yet stabilised
     */
    public Object include(final String path) {  // TODO: Support jar URIs
        try {
            if (path.startsWith("file:/") || path.startsWith("jar:file:/")) {
                return include(new URI(path));
            }
            final File f = new File(path);
            if (f.isAbsolute()) {
                return include(f.toURI());
            }
            
            // Get a reasonable path
            final Sandbox s = getSandbox();
            final URI callerUri = s.loadedFrom();
            // This isn't very elegant but works (in most cases?)...
            final String c = callerUri.toString();
            final int idx = callerUri.toString().lastIndexOf('/');
            final URI reqUri = new URI(c.substring(0, idx)+"/"+path).normalize();
            
            return include(reqUri);
        }
        catch (final URISyntaxException e) {
            e.printStackTrace();
        }
        return null;
    }

    public Object include(final URI uri) {
        if (null == uri) {
            // TODO: Handle better
            return null;
        }
        if (null == uri.getScheme() ||
                (!"file".equals(uri.getScheme()) && !"jar".equals(uri.getScheme()))) {
            throw new IllegalArgumentException("Only file and jar-contained files can be included");
        }

        try {
            return getSandbox().inject(new PluginReader(uri));
        }
        catch (final IOException e) {
            // FIXME: Handle better
            System.err.println("Failed to read " + uri);
        }
        catch (final PluginException e) {
            // FIXME: Handle better
            System.err.println("Exception: " + e.getMessage());
        }
        return null;
    }
}
